# coding: utf-8
import math
import numpy as np
import matplotlib.pyplot as plt
from scipy.integrate import solve_ivp

p = 10
r = 28
b = 8/3

def lorenz_eq(t, y):
    X = y[0]
    Y = y[1]
    Z = y[2]
    dXdt = p * (Y - X)
    dYdt = r*X - X*Z - Y
    dZdt = X*Y - b*Z
    return [dXdt, dYdt, dZdt]

X=1
Y=0
Z=0
time = 0
def single_step(tstep):
    global time,X,Y,Z
    span = (time,time+tstep)
    args=()    
    init_vals = [X, Y, Z]
    res = solve_ivp(lorenz_eq, t_span=span, y0=init_vals, t_eval=None,
                     method='RK45', rtol=1.0e-6, atol=1.0e-12, args=args)
    X = res.y[0,-1]
    Y = res.y[1,-1]
    Z = res.y[2,-1]
    time += tstep
    return res

#####

vertex_source = """#version 330 core
    in vec3 position;
    in vec4 colors;
    out vec4 vertex_colors;

    uniform WindowBlock
    {
        mat4 projection;
        mat4 view;
    } window;

    uniform mat4 model ;

    void main()
    {
        gl_Position = window.projection * window.view * model * vec4(position, 1.0) ;
        vertex_colors = colors;
    }
"""

fragment_source = """#version 330 core
    in vec4 vertex_colors;
    out vec4 final_colors;

    void main()
    {
        final_colors = vertex_colors;
    }
"""

import pyglet
from pyglet.gl import *
from pyglet.math import Mat4, Vec3
import pyglet.window.key 

config = Config(sample_buffers=1, samples=4, depth_size=16, double_buffer=True)
window = pyglet.window.Window(width=1280, height=720, resizable=True)
window.set_caption('Lorenz Model')

batch = pyglet.graphics.Batch()
vert_shader = pyglet.graphics.shader.Shader(vertex_source, 'vertex')
frag_shader = pyglet.graphics.shader.Shader(fragment_source, 'fragment')
shader = pyglet.graphics.shader.ShaderProgram(vert_shader, frag_shader)
shader.use()

model_matrix = Mat4((1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1))

@window.event
def on_draw():
    window.clear()
    batch.draw()

@window.event
def on_resize(width, height):
    window.viewport = (0, 0, width, height)
    window.projection = Mat4.perspective_projection(window.aspect_ratio, z_near=0.1, z_far=400, fov=60)
    return pyglet.event.EVENT_HANDLED

@window.event
def on_mouse_drag(x, y, dx, dy, buttons, modifiers):
    global model_matrix
    model_matrix = Mat4.from_rotation(dx*0.05, Vec3(0, 1, 0))  @ model_matrix
    model_matrix = Mat4.from_rotation(-dy*0.05, Vec3(1, 0, 0)) @ model_matrix
    shader['model'] = model_matrix
    return pyglet.event.EVENT_HANDLED

@window.event
def on_mouse_scroll(x, y, scroll_x, scroll_y):
    global model_matrix
    model_matrix = Mat4.from_translation(Vec3(0, 0, scroll_y))  @ model_matrix
    shader['model'] = model_matrix
    return pyglet.event.EVENT_HANDLED

@window.event
def on_key_press(symbol, modifiers):
    global model_matrix    
    if symbol == pyglet.window.key.R:
        model_matrix = Mat4((1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1))
        shader['model'] = model_matrix
    return pyglet.event.EVENT_HANDLED

def draw_line(shader,x0,y0,z0,x1,y1,z1, rgb):
    vertex_list = shader.vertex_list(2, pyglet.gl.GL_LINES, \
                                     position=('f', (x0, y0, z0, x1, y1, z1)), \
                                     colors=('Bn', (rgb[0], rgb[1], rgb[2], 255,  rgb[0], rgb[1], rgb[2], 255)), \
                                     batch = batch)
    return vertex_list

NLIMIT=100000
XLIST=[ ]
YLIST=[ ]
ZLIST=[ ]
lines = [ ]
def update(deltat):
    global time
    res = single_step(0.01)
    x1 = res.y[0,-1]
    y1 = res.y[1,-1]
    z1 = res.y[2,-1]    
    XLIST.append(x1)
    YLIST.append(y1)
    ZLIST.append(z1)
    if len(XLIST) >= NLIMIT:
        lines[0].delete()
        lines.pop(0)
        XLIST.pop(0)
        YLIST.pop(0)
        ZLIST.pop(0)
    if len(XLIST)>1:
        line = draw_line(shader,XLIST[-2],YLIST[-2],ZLIST[-2], XLIST[-1], YLIST[-1], ZLIST[-1], (255,30,0))
        lines.append(line)

glClearColor(0.15, 0.15, 0.3, 1)
glEnable(GL_DEPTH_TEST)
glEnable(GL_CULL_FACE)
on_resize(*window.size)
glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)

axis_x = draw_line(shader,-100,0,0, +100, 0, 0, (80,80,80))
axis_y = draw_line(shader,0,-100,0, 0, +100, 0, (80,80,80))
axis_z = draw_line(shader,0, 0,-100, 0, 0, +100, (80,80,80))

window.view = window.view.translate((0, 0, -70))

shader['model'] = model_matrix

pyglet.clock.schedule_interval(update,1/120.0)
pyglet.app.run()
